//
//  main.cpp
//  moments
//
//  Created by Vincent Mathieu on 20/09/2019.
//  Copyright © 2019 Vincent Mathieu. All rights reserved.
//

#include "main.hpp"


int main(int argc, const char * argv[]) {
  
  std::complex<double> ur (1,0), ui (0,1);
  std::complex<double> waves[9];
  /*
   * Waves ordering:
   * waves[0]         = S_0 ;
   * waves[1,2,3]     = P_{-1,0,+1} ;
   * waves[4,5,6,7,8] = D_{-2,-1,0,+1,+2}
   */
  
  // random initialization
  printf("\n\nGenerate random waves (only positive reflectivity)\n");
  for (int i=0; i<9; i++) {
    waves[i] = 10.*rand()/RAND_MAX;
    //waves[i] = 1.*i + 2.*ui;
  }
  
  // Compute moments up to L = 4;
  printf("Moments:\n");
  int L,M;
  double mom0[5][5], mom1[5][5], mom2[5][5];
  
  
  for (L=0; L<=4; L++) {
    for (M=0; M<=L; M++) {
      
      mom0[L][M] = moments_H0_SPD(L,M, waves);
      mom1[L][M] = moments_H1_SPD(L,M, waves);
      mom2[L][M] = moments_H2_SPD(L,M, waves);
      
      printf("(L,M) = (%i,%i): H0 = %8.4f ; H1 = %8.4f ; H2 = %8.4f\n",
             L, M, mom0[L][M], mom1[L][M], mom2[L][M]);
    }}
  
  // reconstruct the intensisties from the moments:
  std::complex<double> myYLM[3][5], ZLM[3][5];
  std::complex<double> U1 (0,0), U2 (0,0), UpRe (0,0), UpIm (0,0);
  
  double theta  =   M_PI*rand()/RAND_MAX; // theta € [0,pi]
  double phi    = 2*M_PI*rand()/RAND_MAX; // phi   € [0,2*pi]
  double polPhi = 2*M_PI*rand()/RAND_MAX; // PHI   € [0,2*pi]
  double Pgam   = 1.*rand()/RAND_MAX;        // P_g   € [0,1]
  
  for (L = 0; L<=2  ; L++) {
    for (M = 0; M<=2*L; M++) {
      myYLM[L][M] = Ylm(L,M-L, theta, phi);
      ZLM[L][M]   = myYLM[L][M]*exp(-ui*polPhi);
      U1   += waves[L*L + M]*myYLM[L][M];
      U2   += waves[L*L + M]*conj(myYLM[L][M]);
      UpRe += waves[L*L + M]*real(ZLM[L][M]);
      UpIm += waves[L*L + M]*imag(ZLM[L][M]);
    }}
  
  double I0 = pow(abs(U1),2.) + pow(abs(U2),2.);
  double I1 = -2.*real(U1*conj(U2));
  double I2 = -2.*imag(U1*conj(U2));
  
  // the intensities from moments
  double I0m = 0, I1m = 0, I2m = 0;
  double tau;
  for (L=0; L<=4; L++) {
    for (M=0; M<=L; M++) {
      tau = 2.; if (M==0) { tau = 1.;}
      I0m +=     (2*L+1)/(4.*M_PI)*tau*mom0[L][M]*SmallWignerD(L, M, 0, theta)*cos(M*phi);
      I1m += -1.*(2*L+1)/(4.*M_PI)*tau*mom1[L][M]*SmallWignerD(L, M, 0, theta)*cos(M*phi);
      I2m +=     (2*L+1)/(4.*M_PI)*tau*mom2[L][M]*SmallWignerD(L, M, 0, theta)*sin(M*phi);
    }}
  
  printf("\nRandom angles:  ");
  printf("theta = %f ; phi = %f\n",theta,phi);
  printf("Intensities from waves  : I0 = %f ; I1 = %f ; I2 = %f\n",I0,I1,I2);
  printf("Intensities from moments: I0 = %f ; I1 = %f ; I2 = %f\n",I0m,I1m,I2m);
  
  double Ifullm = I0m - Pgam*I1m*cos(2.*polPhi) - Pgam*I2m*sin(2.*polPhi);
  double Ifull = 2.*(1.-Pgam)*pow(abs(UpIm),2.) + 2.*(1.+Pgam)*pow(abs(UpRe),2.) ;  // only positive refelctivity!!!!
  printf("\nRandom polarization:  ");
  printf("PHI = %f ; P_gamma = %f\n",polPhi,Pgam);
  printf("Full intensity from waves  : I(Omega,PHI) = %f \n",Ifull);
  printf("Full intensity from moments: I(Omega,PHI) = %f \n",Ifullm);
  
  
  
  
  printf("\n");
  return 0;
}



